# -*- coding: utf8 -*-
#
# cacaeditor    Libcaca ascii editor.
#
# Copyright (c) 2011 Alex Foulon <alxf@lavabit.com>
#               All Rights Reserved
#
# This program is free software. It comes without any warranty, to
# the extent permitted by applicable law. You can redistribute it
# and/or modify it under the terms of the Do What The Fuck You Want
# To Public License, Version 2, as published by Sam Hocevar. See
# http://sam.zoy.org/wtfpl/COPYING for more details.
#

""" Libcaca ASCii editor.
"""

#debug
#import pdb

#standard modules
import getopt
import os.path
import sys

#third party module
#catch import error for needed libcaca python bindings
try:
    import caca
    import cacaeditor
    from cacaeditor.gui import App
except ImportError, err:
    sys.stderr.write("FATAL: %s\n" % err)
    sys.exit(127)

#fetch supported formats by libcaca
availables_iformats = [fmt[0] for fmt in caca.get_import_list()]
availables_iformats.pop(0)
iformats_string = ", ".join(availables_iformats)
availables_eformats = [fmt[0] for fmt in caca.get_export_list()]
eformats_string = ", ".join(availables_eformats)

HELP_MSG="""\
Usage: %s [ OPTIONS ] <filename>

Options:
    -c XX, --column=XX,             set the column size (default: 80)
    -d, --debug,                    enable debug messages
    -D mod1,mod2,                   give list of modules to debug (default all)
       --debug-modules=mod1,mod2,
    -h, --help,                     display this message
    -i FORMAT,                      give the format to open filename
       --import-fmt FORMAT,
    -r XX, --row=XX,                set the row size (default: 32)
    -v, --version,                  print the program version

Availables import formats:
  %s

Availables export formats:
  %s

Availables module to debug:
  core, handles, commands, gui, widgets, helpers

""" % (os.path.basename(sys.argv[0]), iformats_string, eformats_string)

VERSION_MSG="""\
%s v%s
""" % (os.path.basename(sys.argv[0]), cacaeditor.__version__)

def main():
    """ Function to handle 'cacaeditor' console script.
    """
    #default options
    open_file = False
    opt_debug = False
    opt_debug_modules = []
    opt_ifmt = None
    opt_column = 80
    opt_row = 32

    #define command options
    shortopts = "c:dD:hi:r:v"
    longopts = [
        'column=',
        'debug',
        'debug-modules=',
        'help',
        'import-fmt=',
        'row=',
        'version',
    ]

    #parse command line with getopt
    try:
        opts, args = getopt.getopt(sys.argv[1:], shortopts, longopts)
    except getopt.GetoptError, e:
        sys.stderr.write("ERROR: %s\n" % e)
        return 127

    for o, a in opts:
        if o in ('-h', '--help'):
            sys.stdout.write("%s" % HELP_MSG)
            return 0
        elif o in ('-v', '--version'):
            sys.stdout.write("%s" % VERSION_MSG)
            return 0
        elif o in ('-d', '--debug'):
            opt_debug = True
        elif o in ('-D', '--debug-modules'):
            opt_debug_modules = a.split(',')
        elif o in ('-c', '--column'):
            opt_column = int(a)
        elif o in ('-r', '--row'):
            opt_row = int(a)
        elif o in ('-i', '--import-fmt'):
            if a in availables_iformats:
                opt_ifmt = a
            else:
                sys.stderr.write("ERROR: Invalid format requested " \
                                 "with -f option.\n")
                return 2

    if args:
        open_file = args[0]

    #launch application
    try:
        editor = App(open_file, opt_ifmt, column=opt_column, row=opt_row,
                     debug=opt_debug, debug_modules=opt_debug_modules)
        editor.main_loop()
    except cacaeditor.AppError, e:
        sys.stderr.write("ERROR: %s\n" % e)
        return 2

    #end application
    return 0

